import React, { useContext } from 'react';
import { Avatar, Typography, Box, Stack, CardHeader, Backdrop, Modal, Fade, Divider, IconButton } from '@mui/material'
import { Close } from '@mui/icons-material'
import { useSelector, connect } from 'react-redux'
import { useHistory } from 'react-router-dom'

import { RecommendedCard } from './recommended';
import ModalWindow from '../structure/modal';
import { ModalForCountsContext, UpdateProfile } from "../../App";

import './style.scss'

import { url } from "../../App"


const style = {
    position: 'absolute',
    top: '50%',
    left: '50%',
    transform: 'translate(-50%, -50%)',
    width: 400,
    height: 'fit-content',
    bgcolor: 'background.paper',
    boxShadow: 24,
    outline: 0,
    borderRadius: 3,
    padding: '5px 0'
};



function AboutMe({ aboutMe = {} }) {
    let history = useHistory()

    const myPostsCount = useSelector(state => state?.promise?.MyPostsCount?.payload)

    function toMyAccount() {
        history.push(`/user/${aboutMe?._id}`)
    }

    // контекст модального окна
    const [modalName, setModalName, modalArray, setModalArray, openModal, setOpenModal, handleOpenModal, handleCloseModal] = useContext(ModalForCountsContext)

    function myFollower() {
        handleOpenModal({ arr: aboutMe?.followers, name: 'Ваши подписчики' })
    }

    function myFollowings() {
        handleOpenModal({ arr: aboutMe?.following, name: 'Ваши подписки' })
    }

    return (
        <Box>
            <CardHeader
                avatar={
                    <Avatar
                        sx={{
                            width: 50,
                            height: 50
                        }}
                        alt={aboutMe?.login}
                        src={url + aboutMe?.avatar?.url}
                    />
                }

                title={
                    <Typography
                        sx={{
                            cursor: 'pointer'
                        }}
                        variant='h6'
                        onClick={toMyAccount}
                    >
                        {aboutMe?.login}
                    </Typography>
                }

                subheader={
                    <Typography
                        variant='subtitle1'
                        color='text.secondary'
                    >
                        {aboutMe?.nick}
                    </Typography>
                }
            />
            <Stack
                direction="row"
                spacing={1}
                padding={2}
                sx={{
                    paddingTop: 0,
                    // flexWrap: 'wrap'
                }}
            >
                <Typography
                    variant='body2'
                    sx={{
                        cursor: 'pointer'
                    }}
                    onClick={toMyAccount}
                >
                    {myPostsCount || '0'} публикаций
                </Typography>

                <Typography
                    variant='body2'
                    sx={{
                        cursor: 'pointer'
                    }}
                    onClick={myFollower}
                >
                    {aboutMe?.followers?.length || '0'} подписчиков
                </Typography>

                <Typography
                    variant='body2'
                    sx={{
                        cursor: 'pointer'
                    }}
                    onClick={myFollowings}
                >
                    {aboutMe?.following?.length || '0'} подписок
                </Typography>
            </Stack>

            {modalArray && <ModalWindow />}
        </Box>
    )
}

export const CAboutMe = connect(state => ({ aboutMe: state?.promise?.AboutMe?.payload }))(AboutMe)