import React, { useEffect, useState } from 'react';
import { connect } from 'react-redux';
import { Box, Divider, Paper } from '@mui/material';
import { styled } from '@mui/material/styles';
import { Container } from '@mui/system';
import Grid2 from '@mui/material/Unstable_Grid2/Grid2';
import { actionAboutMe, actionDownloadFeed } from '../../redux/thunks';
import { CardFeed } from './card';
import { CAboutMe } from './aboutMe';
import Recommended from './recommended';

const Item = styled(Paper)(() => ({
    padding: '0 10px',
    marginBottom: 10,
    borderRadius: 0,
    boxShadow: 'none',
}))

function Feed({ feed, loadFeed }) {
    const [loading, setLoading] = useState(false);
    const [page, setPage] = useState(feed);

    const handleScroll = () => {
        if ((window.innerHeight + window.scrollY) >= document.body.offsetHeight) {
            setPa(dispatch(actionDownloadFeed()))
        }
    };

    useEffect(() => {
        loadFeed();
        window.addEventListener('scroll', handleScroll);
        return () => window.removeEventListener('scroll', handleScroll);
    }, []);

    useEffect(() => {
        if (!loading) return;
        setLoading(true);
        loadFeed(page).then(() => setLoading(false));
    }, [page]);

    const renderFeed = () => {
        if (!feed) return null;
        const filteredFeed = feed.filter(
            (item, index, self) =>
                index === self.findIndex((t) => t._id === item._id)
        );
        return filteredFeed.map((post) => (
            <Item key={post._id}>
                <CardFeed postData={post} />
            </Item>
        ));
    };

    return (
        <Container sx={{ width: '90%', mt: 1 }}>
            <Box sx={{ flexGrow: 1 }}>
                <Grid2 container spacing={2}>
                    <Grid2
                        xs={7}
                        sx={{ display: 'flex', justifyContent: 'center' }}
                    >
                        <Item>{renderFeed()}</Item>
                    </Grid2>
                    <Grid2
                        xs={5}
                        sx={{ display: 'flex', justifyContent: 'center' }}
                    >
                        <Item sx={{ position: 'fixed' }}>
                            <CAboutMe />
                            <Recommended />
                        </Item>
                    </Grid2>
                </Grid2>
            </Box>
        </Container>
    );
}

export const CFeed = connect(
    (state) => ({ feed: state.feed?.MyFeed?.payload }),
    { loadFeed: actionAboutMe }
)(Feed);

