import { url } from "../../App"

import React, { useContext } from 'react';
import { useHistory, useParams } from 'react-router-dom';
import { useDispatch, useSelector } from 'react-redux';

import { CardContent, Typography, Stack, Avatar, Box, Container } from '@mui/material';
import Button from '@mui/material-next/Button';
import { PersonRemoveRounded, PersonAddRounded, ManageAccountsRounded } from '@mui/icons-material';
import { actionUserPageSubscribing } from "../../redux/thunks";

import ModalWindow from '../structure/modal';
import { ModalForCountsContext, UpdateProfile } from "../../App";



// Функция отображения кнопки редактирования своего профиля/отписки или подписки на другого юзера
function UptadeProfileButton({ data }) {
    // console.log('UserData: ', data)

    const history = useHistory()
    const { userId } = useParams()
    const dispatch = useDispatch()

    // контекст обновления профиля
    const [updateProfile, setUpdateProfile] = useContext(UpdateProfile)
    console.log('updateProfile: ', updateProfile)

    // определяем мой id
    const myId = updateProfile?._id

    // определяем всех моих подписчиков
    const myFollowingList = updateProfile?.following?.map(user => user?._id)

    // проверка, является ли пользователь моим подписчиком
    const isFollowing = myFollowingList && (myFollowingList)?.some(item => item === (data?.owner?._id || data?._id))


    // функция подписки/отписки
    function isSubscribing() {

        // при клике на посте
        const newData = {
            ...updateProfile, following: (isFollowing
                ? updateProfile?.following?.filter(item => item?._id !== (data?.owner?._id || data?._id)) // отписка
                // : [...updateProfile?.following, { _id: (data?.owner?._id || data?._id) }] // подписка
                : updateProfile?.following
                    ? [...updateProfile?.following, { _id: (data?.owner?._id || data?._id) }] // подписка если уже есть кто-то в подписках
                    : [{ _id: (data?.owner?._id || data?._id) }] // подписка, если это первая подписка
            )
        }

        dispatch(actionUserPageSubscribing(newData, userId))
    }

    // функция перехода на редактирование профиля
    function upsertProfile() {
        history.push('/updateprofile')
    }


    return (
        <React.Fragment>
            {userId === myId && <Box
                onClick={upsertProfile}
            >
                <Button
                    size="large"
                    variant="elevated"
                    sx={{
                        minWidth: '300px'
                    }}
                    startIcon={
                        <ManageAccountsRounded />
                    }
                    disableRipple
                >
                    <Typography
                        variant='button'
                        display='block'
                    >
                        Редактировать аккаунт
                    </Typography>
                </Button>
            </Box>}
            {myFollowingList?.includes(userId) && <Box
                onClick={isSubscribing}
            >
                <Button
                    size="large"
                    variant="elevated"
                    sx={{
                        minWidth: '300px'
                    }}
                    startIcon={
                        <PersonRemoveRounded />
                    }
                    disableRipple
                >
                    <Typography
                        variant='button'
                        display='block'
                    >
                        Отписаться
                    </Typography>
                </Button>
            </Box>}
            {(userId !== myId && !myFollowingList?.includes(userId)) && <Box
                onClick={isSubscribing}
            >
                <Button
                    size="large"
                    variant="filledTonal"
                    sx={{
                        minWidth: '300px'
                    }}
                    startIcon={
                        <PersonAddRounded />
                    }
                    disableRipple
                >
                    <Typography
                        variant='button'
                        display='block'
                    >
                        Подписаться
                    </Typography>
                </Button>
            </Box>}
        </React.Fragment>

    )
}



// блок пользовательских данных
export default function BasicCard({ userData }) {
    // console.log('userData: ', userData)

    // определяем количество постов пользоваеля
    const userPostsCount = useSelector(state => state?.promise?.UserPostsCount?.payload)

    // контекст модального окна
    const [modalName, setModalName, modalArray, setModalArray, openModal, setOpenModal, handleOpenModal, handleCloseModal] = useContext(ModalForCountsContext)

    function userFollowers() {
        handleOpenModal({
            arr: userData?.followers, name: `Подписчики ${userData?.login}`
        })
    }

    function userFollowing() {
        handleOpenModal({
            arr: userData?.following, name: `Подписки ${userData?.login}`
        })
    }


    return (
        <Container sx={{
            display: 'flex',
            alignItems: 'center'
        }}
        >
            <Avatar
                alt={userData?.login}
                src={url + userData?.avatar?.url}
                sx={{
                    width: 130,
                    height: 130
                }}
            />

            <Box
                sx={{
                    display: 'flex',
                    flexDirection: 'column',
                    marginLeft: '50px '
                }}
            >
                <CardContent sx={{
                    flex: '1 0 auto'
                }}
                >
                    <Stack
                        sx={{
                            justifyContent: 'space-between'
                        }}
                        direction="row"
                        spacing={2}
                    >
                        <Typography
                            component="div"
                            variant="h4"
                        >
                            {userData?.login}
                        </Typography>

                        <UptadeProfileButton data={userData} />
                    </Stack>

                    <Typography
                        variant="subtitle1"
                        color="text.secondary"
                        component="div"
                    >
                        {userData?.nick || ''}
                    </Typography>
                </CardContent>

                <Stack
                    direction="row"
                    spacing={5}
                    padding={2}
                    paddingTop={1}
                >
                    <Typography
                        variant="subtitle1"
                    >
                        {userPostsCount || '0'} публикаций
                    </Typography>

                    <Typography
                        sx={{
                            cursor: 'pointer'
                        }}
                        variant="subtitle1"
                        onClick={userFollowers}
                    >
                        {userData?.followers?.length || '0'} подписчиков
                    </Typography>

                    <Typography
                        sx={{
                            cursor: 'pointer'
                        }}
                        variant="subtitle1"
                        onClick={userFollowing}
                    >
                        {userData?.following?.length || '0'} подписок
                    </Typography>
                </Stack>
            </Box>

            {modalArray && <ModalWindow />}
        </Container>
    )
}